using System.ComponentModel;
using System.Security.Permissions;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using WebMoney.Types;
using System.Text;

namespace WMSigner.WebControls
{
    [ToolboxData("<{0}:ClickAndBuy runat=server />")]
    [Designer(typeof(Design.CntrDesigner))]
    [DefaultEvent("Click")]
    [AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal), AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    public class ClickAndBuy : WebControl
    {
        private const string WMKPayLink = "wmk:paylink";
        private const string WMLPayLinkRus = "https://light.webmoney.ru/pci.aspx";
        private const string WMLPayLinkEng = "https://light.wmtransfer.com/pci.aspx";

        private KeeperMode _keeperMode;
        private string _name;
        private string _url;
        private Purse _purse;
        private Amount _amount;
        private bool _isPost;
        private string _desc;
        private bool _isTestMode;

        [Bindable(false)]
        [Category("Command")]
        [Localizable(false)]
        [DefaultValue(KeeperMode.Classic)]
        public KeeperMode KeeperMode
        {
            get
            {
                return _keeperMode;
            }
            set
            {
                _keeperMode = value;
            }
        }

        [Bindable(false)]
        [Category("Command")]
        [Localizable(false)]
        public string Name
        {
            get
            {
                return _name;
            }
            set
            {
                _name = value;
            }
        }

        [Bindable(false)]
        [Category("Command")]
        [Localizable(false)]
        public string Url
        {
            get
            {
                return _url;
            }
            set
            {
                _url = value;
            }
        }

        [Bindable(false)]
        [Category("Command")]
        [Localizable(false)]
        [DefaultValue("Z000000000000")]
        public Purse Purse
        {
            get
            {
                return _purse;
            }
            set
            {
                _purse = value;
            }
        }

        [Bindable(false)]
        [Category("Command")]
        [Localizable(false)]
        [DefaultValue(0)]
        public Amount Amount
        {
            get
            {
                return _amount;
            }
            set
            {
                _amount = value;
            }
        }

        [Bindable(false)]
        [Category("Command")]
        [Localizable(false)]
        [DefaultValue(true)]
        public bool IsPost
        {
            get
            {
                return _isPost;
            }
            set
            {
                _isPost = value;
            }
        }

        [Bindable(false)]
        [Category("Command")]
        [Localizable(true)]
        public string Desc
        {
            get
            {
                return _desc;
            }
            set
            {
                _desc = value;
            }
        }

        [Bindable(false)]
        [Category("Command")]
        [Localizable(false)]
        [DefaultValue(false)]
        public bool IsTestMode
        {
            get
            {
                return _isTestMode;
            }
            set
            {
                _isTestMode = value;
            }
        }

        protected override void Render(HtmlTextWriter output)
        {
            string keeperMode;

            switch(_keeperMode)
            {
                case KeeperMode.LightEn:
                    keeperMode = WMLPayLinkEng.ToString();
                    break;
                case KeeperMode.LightRu:
                    keeperMode = WMLPayLinkRus.ToString();
                    break;
                default:
                    keeperMode = WMKPayLink.ToString();
                    break;
            }

            StringBuilder link = new StringBuilder();

            link.Append(keeperMode);

            link.Append(string.Format(null, "?url=<{0}>", _url));
            link.Append(string.Format(null, "&purse={0}", _purse));
            link.Append(string.Format(null, "&amount={0}", _amount));
            link.Append(string.Format(null, "&method={0}", (_isPost) ? "POST" : "GET"));
            link.Append(string.Format(null, "&desc={0}", _desc));
            link.Append(string.Format(null, "&mode={0}", (_isTestMode) ? "test" : string.Empty));

            output.Write(string.Format(null, "<a href=\"{0}\">{1}</a>", link, _name));
        }
    }
}
