using System;
using System.Globalization;
using System.ComponentModel;
using WMSigner.WebControls.Types;

namespace WebMoney.Types
{
    [
    TypeConverter(typeof(AmountConverter))
    ]
    public struct Amount
    {
        private const double MAX_VALUE = 9223372036854.77;
        const string SEP = "."; //   
        double _value;

        public static explicit operator Amount(double value)
        {
            return new Amount(value);
        }

        public static implicit operator double(Amount value)
        {
            return value._value;
        }

        public static explicit operator Amount(string value)
        {
            Amount result = new Amount();
            result.FromString(value);
            return result;
        }

        public static bool operator ==(Amount lhs, Amount rhs)
        {
            return Math.Round(lhs, 2) == Math.Round(rhs, 2);
        }

        public static bool operator != (Amount lhs, Amount rhs)
        {
            return !(lhs == rhs);
        }

        /// <summary>
        ///   Amount
        /// </summary>
        /// <param name="value"> double</param>
        Amount(double value)
        {
            if (value > MAX_VALUE)
                throw new OverflowException();

            _value = value;
        }

        /// <summary>
        ///  Amount    
        /// </summary>
        /// <returns>Amount   </returns>
        public override string ToString()
        {
            string result;

            NumberFormatInfo amountFormat = new NumberFormatInfo();
            amountFormat.CurrencyDecimalSeparator = SEP;
            amountFormat.CurrencySymbol = string.Empty;
            amountFormat.CurrencyGroupSeparator = string.Empty;

            result = _value.ToString("C", amountFormat); //       

            int resultLen = result.Length;

            if ('0' == result[resultLen - 1]) //      '0'
            {
                resultLen -= 1; //         

                if ('0' == result[resultLen - 1]) //     '0'
                    resultLen -= 2; //            
            }

            return result.Substring(0, resultLen);
        }

        public void FromString(string value)
        {
            int sepInd = value.IndexOf(SEP);
            int secondSepInd = value.IndexOf(SEP, sepInd + 1);

            if (secondSepInd >= 0)
                value = value.Remove(secondSepInd, 1);

            NumberFormatInfo numberFormat = new NumberFormatInfo();
            numberFormat.NumberDecimalSeparator = SEP;

            double temp = double.Parse(value, NumberStyles.Number, numberFormat);

            if (temp > MAX_VALUE)
                throw new OverflowException();

            _value = temp;
        }
    }
}