using System;
using System.ComponentModel;
using WMSigner.WebControls.Types;

namespace WebMoney.Types
{
    /// <summary>
    ///    WebMoney
    /// </summary>
    [
    TypeConverter(typeof(PurseConverter))
    ]
    public struct Purse
    {
        const int NUM_LEN = 12; //    (,   )

        PurseType _type;
        ulong _number;

        /// <summary>
        ///  
        /// </summary>
        public PurseType Type
        {
            get
            {
                return _type;
            }
        }

        /// <summary>
        ///   ( )
        /// </summary>
        public ulong Number
        {
            get
            {
                return _number;
            }
        }

        public static explicit operator Purse(string value)
        {
            return new Purse(value);
        }

        /// <summary>
        ///   Purse
        /// </summary>
        /// <param name="purseStr">   "Z123456789012"</param>
        Purse(string purseStr)
        {
            _type = getType(purseStr);
            _number = getNumber(purseStr);
        }

        public Purse(PurseType type, ulong number)
        {
            _type = type;
            _number = number;
        }

        /// <summary>
        ///  Purse   
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            char letter = typeToLetter(_type);
            return letter + _number.ToString("000000000000");
        }

        static PurseType getType(string purseStr)
        {
            char letter = purseStr.Substring(0, 1)[0];
            return letterToType(letter);
        }

        static ulong getNumber(string purseStr)
        {
            string numberStr = purseStr.Substring(1);

            if (NUM_LEN != numberStr.Length)
                throw new ArgumentOutOfRangeException();

            return ulong.Parse(numberStr);
        }

        static PurseType letterToType(char letter)
        {
            PurseType type;

            switch (letter)
            {
                case 'Z':
                    type = PurseType.WMZ;
                    break;
                case 'E':
                    type = PurseType.WME;
                    break;
                case 'R':
                    type = PurseType.WMR;
                    break;
                case 'U':
                    type = PurseType.WMU;
                    break;
                case 'B':
                    type = PurseType.WMB;
                    break;
                case 'Y':
                    type = PurseType.WMY;
                    break;
                case 'G':
                    type = PurseType.WMG;
                    break;
                default:
                    throw new NotSupportedException("Wrong purse type");
            }

            return type;
        }

        static char typeToLetter(PurseType type)
        {
            char letter;

            switch (type)
            {
                case PurseType.WMZ:
                    letter = 'Z';
                    break;
                case PurseType.WME:
                    letter = 'E';
                    break;
                case PurseType.WMY:
                    letter = 'Y';
                    break;
                case PurseType.WMR:
                    letter = 'R';
                    break;
                case PurseType.WMU:
                    letter = 'U';
                    break;
                case PurseType.WMB:
                    letter = 'B';
                    break;
                case PurseType.WMG:
                    letter = 'G';
                    break;
                default:
                    throw new NotSupportedException("Wrong purse type");

            }

            return letter;
        }

        #region IStringConvertible Members

        public void FromString(string purseStr)
        {
            _type = getType(purseStr);
            _number = getNumber(purseStr);
        }

        #endregion
    }
}